@@ -1,7 +1,7 @@
 /*
  * MIT License
  *
- * Copyright (c) 2014-2025 Patrizio Bekerle -- <patrizio@bekerle.com>
+ * Copyright (c) 2014-2026 Patrizio Bekerle -- <patrizio@bekerle.com>
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -283,6 +283,17 @@ bool QMarkdownTextEdit::eventFilter(QObject *obj, QEvent *event) {
             return bracketClosingCheck(QLatin1Char('['), QLatin1Char(']'));
         } else if (keyEvent->key() == Qt::Key_Greater) {
             return bracketClosingCheck(QLatin1Char('<'), QLatin1Char('>'));
+        } else if ((keyEvent->key() == Qt::Key_Return ||
+                    keyEvent->key() == Qt::Key_Enter) &&
+                   !isReadOnly() &&
+                   keyEvent->modifiers().testFlag(Qt::ShiftModifier) &&
+                   keyEvent->modifiers().testFlag(Qt::ControlModifier)) {
+            QTextCursor cursor = this->textCursor();
+            cursor.movePosition(QTextCursor::StartOfBlock);
+            cursor.insertText(QStringLiteral("\n"));
+            cursor.movePosition(QTextCursor::Up);
+            setTextCursor(cursor);
+            return true;
         } else if ((keyEvent->key() == Qt::Key_Return ||
                     keyEvent->key() == Qt::Key_Enter) &&
                    !isReadOnly() &&
@@ -1220,6 +1231,13 @@ bool QMarkdownTextEdit::openLinkAtCursorPosition() {
 
     if ((url.isValid() && isValidUrl(urlString)) || isRelativeFileUrl ||
         isLegacyAttachmentUrl) {
+        // Check if URL matches any ignored regular expressions
+        for (const QRegularExpression &regex : _ignoredClickUrlRegexps) {
+            if (regex.match(urlString).hasMatch()) {
+                return true; // URL matches ignored regex, don't open it
+            }
+        }
+
         // ignore some schemata
         if (!(_ignoredClickUrlSchemata.contains(url.scheme()) ||
               isRelativeFileUrl || isLegacyAttachmentUrl)) {
@@ -1277,13 +1295,22 @@ QPlainTextEditSearchWidget *QMarkdownTextEdit::searchWidget() {
 
 /**
  * @brief Sets url schemata that will be ignored when clicked on
- * @param urlSchemes
+ * @param ignoredUrlSchemata
  */
 void QMarkdownTextEdit::setIgnoredClickUrlSchemata(
     QStringList ignoredUrlSchemata) {
     _ignoredClickUrlSchemata = std::move(ignoredUrlSchemata);
 }
 
+/**
+ * @brief Sets url regular expressions that will be ignored when clicked on
+ * @param ignoredClickUrlRegexps
+ */
+void QMarkdownTextEdit::setIgnoredClickUrlRegexps(
+    QList<QRegularExpression> ignoredClickUrlRegexps) {
+    _ignoredClickUrlRegexps = std::move(ignoredClickUrlRegexps);
+}
+
 /**
  * @brief Returns a map of parsed Markdown urls with their link texts as key
  *
