@@ -568,6 +568,8 @@ class MainWindow : public QMainWindow {
 
     void on_actionFind_action_triggered();
 
+    void on_actionRedo_action_triggered();
+
     void releaseDockWidgetSizes();
 
     void on_actionInsert_table_triggered();
@@ -881,6 +883,7 @@ class MainWindow : public QMainWindow {
     bool _isMaximizedBeforeFullScreen = false;
     bool _isMinimizedBeforeFullScreen = false;
     WaitingSpinnerWidget *_openAiActivitySpinner = nullptr;
+    QAction *_lastTriggeredAction = nullptr;
 
     void initializeOpenAiActivitySpinner();
 
@@ -1134,7 +1137,8 @@ class MainWindow : public QMainWindow {
 
     void initFakeVim(QOwnNotesMarkdownTextEdit *noteTextEdit);
 
-    void openNotesContextMenu(const QPoint globalPos, bool multiNoteMenuEntriesOnly = false);
+    void openNotesContextMenu(const QPoint globalPos, bool hasNotes = true,
+                              bool hasFolders = false);
 
     void updateCurrentNoteTextHash();
 
@@ -1172,6 +1176,7 @@ class MainWindow : public QMainWindow {
     static void handleDockWidgetLocking(QDockWidget *dockWidget);
     void setupNoteRelationScene();
     void updateNoteGraphicsView();
+    bool noteDirectoryWatcherAddPath(const QString &path);
     void addDirectoryToDirectoryWatcher(const QString &path);
     bool nextCloudDeckCheck();
 };
